@@ -37,7 +37,8 @@ from ..piv import (
     UnsupportedAlgorithm, WrongPin, WrongPuk)
 from ..driver_ccid import APDUError, SW
 from .util import (
-    click_force_option, click_postpone_execution, click_callback,
+    click_force_option, click_format_option,
+    click_postpone_execution, click_callback,
     prompt_for_touch, UpperCaseChoice)
 from cryptography import x509
 from cryptography.hazmat.primitives import hashes, serialization
@@ -59,16 +60,6 @@ def click_parse_piv_slot(ctx, param, val):
         raise ValueError(val)
 
 
-@click_callback()
-def click_parse_format(ctx, param, val):
-    if val == 'PEM':
-        return serialization.Encoding.PEM
-    elif val == 'DER':
-        return serialization.Encoding.DER
-    else:
-        raise ValueError(val)
-
-
 @click_callback()
 def click_parse_management_key(ctx, param, val):
     try:
@@ -88,10 +79,6 @@ click_management_key_option = click.option(
     callback=click_parse_management_key)
 click_pin_option = click.option(
     '-P', '--pin', help='PIN code.')
-click_format_option = click.option(
-    '-F', '--format',
-    type=UpperCaseChoice(['PEM', 'DER']), default='PEM', show_default=True,
-    help='Encoding format.', callback=click_parse_format)
 click_pin_policy_option = click.option(
     '--pin-policy',
     type=UpperCaseChoice(['DEFAULT', 'NEVER', 'ONCE', 'ALWAYS']),
@@ -168,18 +155,46 @@ def info(ctx):
 
     for (slot, cert) in controller.list_certificates().items():
         click.echo('Slot %02x:' % slot)
-        click.echo('\tAlgorithm:\t%s' % ALGO.from_public_key(cert.public_key())
-                   .name)
-        cn = cert.subject.get_attributes_for_oid(x509.NameOID.COMMON_NAME)
-        cn = cn[0].value if len(cn) > 0 else 'None'
-        click.echo('\tSubject CN:\t%s' % cn)
-        cn = cert.issuer.get_attributes_for_oid(x509.NameOID.COMMON_NAME)
-        cn = cn[0].value if len(cn) > 0 else 'None'
-        click.echo('\tIssuer CN:\t%s' % cn)
-        click.echo('\tFingerprint:\t%s' % b2a_hex(
-            cert.fingerprint(hashes.SHA256())).decode('ascii'))
-        click.echo('\tNot before:\t%s' % cert.not_valid_before)
-        click.echo('\tNot after:\t%s' % cert.not_valid_after)
+
+        try:
+            # Try to read out full DN, fallback to only CN.
+            # Support for DN was added in crytography 2.5
+            subject_dn = cert.subject.rfc4514_string()
+            issuer_dn = cert.issuer.rfc4514_string()
+            print_dn = True
+        except AttributeError:
+            print_dn = False
+            logger.debug('Failed to read DN, falling back to only CNs')
+            subject_cn = cert.subject.get_attributes_for_oid(
+                x509.NameOID.COMMON_NAME)
+            subject_cn = subject_cn[0].value if subject_cn else 'None'
+            issuer_cn = cert.issuer.get_attributes_for_oid(
+                x509.NameOID.COMMON_NAME)
+            issuer_cn = issuer_cn[0].value if issuer_cn else 'None'
+        except ValueError as e:
+            # Malformed certificates may throw ValueError
+            logger.debug('Failed parsing certificate', exc_info=e)
+            click.echo('\tMalformed certificate: {}'.format(e))
+            continue
+
+        fingerprint = b2a_hex(cert.fingerprint(hashes.SHA256())).decode('ascii')
+        algo = ALGO.from_public_key(cert.public_key())
+        serial = cert.serial_number
+        not_before = cert.not_valid_before
+        not_after = cert.not_valid_after
+
+        # Print out everything
+        click.echo('\tAlgorithm:\t%s' % algo.name)
+        if print_dn:
+            click.echo('\tSubject DN:\t%s' % subject_dn)
+            click.echo('\tIssuer DN:\t%s' % issuer_dn)
+        else:
+            click.echo('\tSubject CN:\t%s' % subject_cn)
+            click.echo('\tIssuer CN:\t%s' % issuer_cn)
+        click.echo('\tSerial:\t\t%s' % serial)
+        click.echo('\tFingerprint:\t%s' % fingerprint)
+        click.echo('\tNot before:\t%s' % not_before)
+        click.echo('\tNot after:\t%s' % not_after)
 
 
 @piv.command()
@@ -274,10 +289,8 @@ def generate_key(
 @click.option(
     '-p', '--password', help='A password may be needed to decrypt the data.')
 @click.option(
-    '--no-verify', 'verify', is_flag=True, default=False,
-    callback=lambda ctx, param, value: not value,
-    help='Skip verifying that the certificate matches the private key in the '
-         'slot.')
+    '-v', '--verify', is_flag=True,
+    help='Verify that the certificate matches the private key in the slot.')
 @click.argument('cert', type=click.File('rb'), metavar='CERTIFICATE')
 def import_certificate(
         ctx, slot, management_key, pin, cert, password, verify):
@@ -324,7 +337,9 @@ def import_certificate(
 
     def do_import(retry=True):
         try:
-            controller.import_certificate(slot, cert_to_import, verify=verify)
+            controller.import_certificate(
+                slot, cert_to_import, verify=verify,
+                touch_callback=prompt_for_touch)
 
         except KeypairMismatch:
             ctx.fail('This certificate is not tied to the private key in the '
@@ -521,7 +536,7 @@ def set_pin_retries(ctx, management_key, pin, pin_retries, puk_retries, force):
 @click.argument('public-key', type=click.File('rb'), metavar='PUBLIC-KEY')
 @click.option(
     '-s', '--subject',
-    help='A subject name for the certificate.', required=True)
+    help='Subject common name (CN) for the certificate.', required=True)
 @click.option(
     '-d', '--valid-days',
     help='Number of days until the certificate expires.',
@@ -568,7 +583,8 @@ def generate_certificate(
 @click.argument('csr-output', type=click.File('wb'), metavar='CSR')
 @click.option(
     '-s', '--subject',
-    help='A subject name for the requested certificate.', required=True)
+    help='Subject common name (CN) for the requested certificate.',
+    required=True)
 def generate_certificate_signing_request(
         ctx, slot, pin, public_key, csr_output, subject):
     """
@@ -622,17 +638,26 @@ def change_pin(ctx, pin, new_pin):
     """
     Change the PIN code.
 
-    The PIN can be up to 8 characters long, and supports any type of
+    The PIN must be between 6 and 8 characters long, and supports any type of
     alphanumeric characters. For cross-platform compatibility,
-    a PIN of 6 - 8 numeric digits is recommended.
+    numeric digits are recommended.
     """
+
     controller = ctx.obj['controller']
+
     if not pin:
         pin = _prompt_pin(ctx, prompt='Enter your current PIN')
     if not new_pin:
         new_pin = click.prompt(
             'Enter your new PIN', default='', hide_input=True,
             show_default=False, confirmation_prompt=True, err=True)
+
+    if not _valid_pin_length(pin):
+        ctx.fail('Current PIN must be between 6 and 8 characters long.')
+
+    if not _valid_pin_length(new_pin):
+        ctx.fail('New PIN must be between 6 and 8 characters long.')
+
     try:
         controller.change_pin(pin, new_pin)
         click.echo('New PIN set.')
@@ -656,6 +681,8 @@ def change_puk(ctx, puk, new_puk):
     Change the PUK code.
 
     If the PIN is lost or blocked it can be reset using a PUK.
+    The PUK must be between 6 and 8 characters long, and supports any type of
+    alphanumeric characters.
     """
     controller = ctx.obj['controller']
     if not puk:
@@ -666,6 +693,12 @@ def change_puk(ctx, puk, new_puk):
             show_default=False, confirmation_prompt=True,
             err=True)
 
+    if not _valid_pin_length(puk):
+        ctx.fail('Current PUK must be between 6 and 8 characters long.')
+
+    if not _valid_pin_length(new_puk):
+        ctx.fail('New PUK must be between 6 and 8 characters long.')
+
     try:
         controller.change_puk(puk, new_puk)
         click.echo('New PUK set.')
@@ -793,6 +826,77 @@ def unblock_pin(ctx, puk, new_pin):
     controller.unblock_pin(puk, new_pin)
 
 
+@piv.command('read-object')
+@click_pin_option
+@click.pass_context
+@click.argument(
+    'object-id',
+    callback=lambda ctx, param, value: int(value, 16),
+    metavar='OBJECT-ID')
+def read_object(ctx, pin, object_id):
+    """
+    Read arbitrary PIV object.
+
+    Read PIV object by providing the object id.
+
+    \b
+    OBJECT-ID       Id of PIV object in HEX.
+    """
+
+    controller = ctx.obj['controller']
+
+    def do_read_object(retry=True):
+        try:
+            click.echo(controller.get_data(object_id))
+        except APDUError as e:
+            if e.sw == SW.NOT_FOUND:
+                ctx.fail('No data found.')
+            elif e.sw == SW.SECURITY_CONDITION_NOT_SATISFIED:
+                _verify_pin(ctx, controller, pin)
+                do_read_object(retry=False)
+            else:
+                raise
+
+    do_read_object()
+
+
+@piv.command('write-object')
+@click_pin_option
+@click_management_key_option
+@click.pass_context
+@click.argument(
+    'object-id',
+    callback=lambda ctx, param, value: int(value, 16),
+    metavar='OBJECT-ID')
+@click.argument('data', type=click.File('rb'), metavar='DATA')
+def write_object(ctx, pin, management_key, object_id, data):
+    """
+    Write an arbitrary PIV object.
+
+    Write a PIV object by providing the object id.
+    Yubico writable PIV objects are available in
+    the range 5f0000 - 5fffff.
+
+    \b
+    OBJECT-ID      Id of PIV object in HEX.
+    DATA           File containing the data to be written. Use '-' to use stdin.
+    """
+
+    controller = ctx.obj['controller']
+    _ensure_authenticated(ctx, controller, pin, management_key)
+
+    def do_write_object(retry=True):
+        try:
+            controller.put_data(object_id, data.read())
+        except APDUError as e:
+            logger.debug('Failed writing object', exc_info=e)
+            if e.sw == SW.INCORRECT_PARAMETERS:
+                ctx.fail('Something went wrong, is the object id valid?')
+            raise
+
+    do_write_object()
+
+
 def _prompt_management_key(
         ctx, prompt='Enter a management key [blank to use default key]'):
     management_key = click.prompt(
@@ -810,6 +914,10 @@ def _prompt_pin(ctx, prompt='Enter PIN'):
         prompt, default='', hide_input=True, show_default=False, err=True)
 
 
+def _valid_pin_length(pin):
+    return 6 <= len(pin) <= 8
+
+
 def _ensure_authenticated(
         ctx, controller, pin=None, management_key=None,
         require_pin_and_key=False,
