@@ -27,7 +27,7 @@
 
 from __future__ import absolute_import
 
-from .util import PID, TRANSPORT, Mode
+from .util import PID, TRANSPORT, Mode, YUBIKEY
 from .device import YubiKey
 from .driver_ccid import open_devices as open_ccid
 from .driver_fido import open_devices as open_fido
@@ -70,6 +70,15 @@ class Descriptor(object):
     def key_type(self):
         return self._key_type
 
+    @property
+    def name(self):
+        if self.key_type == YUBIKEY.SKY and self.version < (5, 0, 0):
+            return 'FIDO U2F Security Key'
+        elif self.key_type == YUBIKEY.YK4 and self.version >= (5, 0, 0):
+            return 'YubiKey 5'
+        else:
+            return self.key_type.value
+
     def open_device(self, transports=sum(TRANSPORT), serial=None, attempts=10):
         self._logger.debug('transports: 0x%x, self.mode.transports: 0x%x',
                            transports, self.mode.transports)
@@ -174,12 +183,12 @@ def _open_driver(transports, serial, key_type, mode, attempts):
                              key_type, dev.driver.key_type)
                 del dev
                 continue
-            return dev.driver
             if mode is not None and dev.driver.mode != mode:
                 logger.debug('Mode does not match. Want: %s, got: %s',
                              mode, dev.driver.mode)
                 del dev
                 continue
+            return dev.driver
         #  Wait a little before trying again.
         logger.debug('Sleeping for %f s', sleep_time)
         time.sleep(sleep_time)
