@@ -231,10 +231,10 @@ class YubiKey(object):
                 else:  # Assume base capabilities
                     logger.debug('CCID not available, guess capabilities')
                     usb_supported = _NEO_BASE_CAPABILITIES
-                    if TRANSPORT.has(self.mode.transports, TRANSPORT.FIDO) \
-                            or (version and version >= (3, 3, 0)):
-                        usb_supported |= APPLICATION.U2F
-
+                # NEO over 3.3.0 have U2F (which might be blocked by OS)
+                if TRANSPORT.has(self.mode.transports, TRANSPORT.FIDO) \
+                        or (version and version >= (3, 3, 0)):
+                    usb_supported |= APPLICATION.U2F
                 config._set(TAG.USB_SUPPORTED, usb_supported)
                 config._set(TAG.NFC_SUPPORTED, usb_supported)
                 config._set(TAG.NFC_ENABLED, usb_supported)
@@ -249,15 +249,19 @@ class YubiKey(object):
                 self._can_mode_switch = False
 
         # Fix usb_enabled
-        usb_enabled = config.usb_enabled or config.usb_supported
-        if not TRANSPORT.has(self.mode.transports, TRANSPORT.OTP):
-            usb_enabled &= ~APPLICATION.OTP
-        if not TRANSPORT.has(self.mode.transports, TRANSPORT.FIDO):
-            usb_enabled &= ~(APPLICATION.U2F | APPLICATION.FIDO2)
-        if not TRANSPORT.has(self.mode.transports, TRANSPORT.CCID):
-            usb_enabled &= ~(TRANSPORT.CCID | APPLICATION.OATH |
-                             APPLICATION.OPGP | APPLICATION.PIV)
-        config._set(TAG.USB_ENABLED, usb_enabled)
+        if not config.usb_enabled:
+            usb_enabled = config.usb_supported
+            if not TRANSPORT.has(self.mode.transports, TRANSPORT.OTP):
+                usb_enabled &= ~APPLICATION.OTP
+            if not TRANSPORT.has(self.mode.transports, TRANSPORT.FIDO):
+                usb_enabled &= ~(APPLICATION.U2F | APPLICATION.FIDO2)
+            if not TRANSPORT.has(self.mode.transports, TRANSPORT.CCID):
+                usb_enabled &= ~(
+                    TRANSPORT.CCID |
+                    APPLICATION.OATH |
+                    APPLICATION.OPGP |
+                    APPLICATION.PIV)
+            config._set(TAG.USB_ENABLED, usb_enabled)
 
         # Workaround for invalid configurations.
         # Assume all form factors except USB_A_KEYCHAIN
@@ -266,6 +270,7 @@ class YubiKey(object):
                 FORM_FACTOR.USB_A_NANO,
                 FORM_FACTOR.USB_C_KEYCHAIN,
                 FORM_FACTOR.USB_C_NANO,
+                FORM_FACTOR.USB_C_LIGHTNING
                 ):
             config._set(TAG.NFC_SUPPORTED, 0)
             config._set(TAG.NFC_ENABLED, 0)
@@ -277,14 +282,17 @@ class YubiKey(object):
             if not APPLICATION.has(config.usb_supported, APPLICATION.FIDO2):
                 logger.debug('SKY has no FIDO2, SKY 1')
                 self.device_name = 'FIDO U2F Security Key'  # SKY 1
+            if config.nfc_supported:
+                self.device_name = 'Security Key NFC'
         elif self._key_type == YUBIKEY.YK4:
-            if (5, 1, 0) > self.version >= (5, 0, 0):
+            if (5, 0, 0) <= self.version < (5, 1, 0) or \
+                    self.version in [(5, 2, 0), (5, 2, 1), (5, 2, 2)]:
                 self.device_name = 'YubiKey Preview'
             elif self.version >= (5, 1, 0):
                 logger.debug('Identified YubiKey 5')
                 self.device_name = 'YubiKey 5'
-                if config.form_factor == FORM_FACTOR.USB_A_KEYCHAIN \
-                    and not config.nfc_supported:
+                if (config.form_factor == FORM_FACTOR.USB_A_KEYCHAIN
+                        and not config.nfc_supported):
                     self.device_name += 'A'
                 elif config.form_factor == FORM_FACTOR.USB_A_KEYCHAIN:
                     self.device_name += ' NFC'
@@ -339,7 +347,11 @@ class YubiKey(object):
 
     @property
     def is_fips(self):
-        return (4, 4, 0) <= self.version < (4, 5, 0)
+        return YubiKey.is_fips_version(self.version)
+
+    @staticmethod
+    def is_fips_version(version):
+        return (4, 4, 0) <= version < (4, 5, 0)
 
     @mode.setter
     def mode(self, mode):
